/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.server;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Connection;

import de.frankmuenster.jameica.depot.JaDepot;
import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.Version;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.sql.version.UpdateProvider;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * Implementiert den UpdateProvdider der Jamaica SQL-Engine
 * 
 * @author munstfr
 * @version 1.0
 * 
 */
public class JaDepotUpdateProvider implements UpdateProvider {

	/**
	 * Die Version des Updates
	 */
	private final transient Version version;

	/**
	 * Das Datenbank-Connection Objekt
	 */
	private final transient Connection conn;

	/**
	 * Plugin Manifest
	 */
	private final transient Manifest manifest;

	/**
	 * Das Plugin-Resource-Ojekt
	 */
	private final transient PluginResources res;

	/**
	 * Geschtzer Konstruktor der Klasse
	 * 
	 * @param conn
	 *            das DB-Verbindungs-Objekt
	 * @param version
	 *            die Versionsinformationen
	 */
	protected JaDepotUpdateProvider(final Connection conn, final Version version) {
		this.conn = conn;
		this.version = version;

		final AbstractPlugin abstractPlugin = Application.getPluginLoader()
				.getPlugin(JaDepot.class);
		this.manifest = abstractPlugin.getManifest();
		this.res = abstractPlugin.getResources();
	}

	/**
	 * @see UpdateProvider#getConnection()
	 */
	@Override
	public Connection getConnection() throws ApplicationException {
		return this.conn;
	}

	/**
	 * Liefert das PluginResource Objekt.
	 * 
	 * @return das PluginResource Objekt.
	 */
	public PluginResources getResources() {
		return this.res;
	}

	/**
	 * @see UpdateProvider#getCurrentVersion()
	 */
	@Override
	public int getCurrentVersion() throws ApplicationException {
		try {
			return this.version.getVersion();
		} catch (RemoteException re) {
			Logger.error(Settings.getI18n().tr(
					"JaDepotUpdateProvider.readVersionError")); //$NON-NLS-1$
			throw (ApplicationException) new ApplicationException(Settings
					.getI18n().tr(
							Settings.getI18n().tr(
									"JaDepotUpdateProvider.readVersionError"))) //$NON-NLS-1$
					.initCause(re);
		}
	}

	/**
	 * @see UpdateProvider#getProgressMonitor()
	 */
	@Override
	public ProgressMonitor getProgressMonitor() {
		// Liefert den Splashscreen oder im Servermode einen
		// Pseudo-Monitor.
		return Application.getController().getApplicationCallback()
				.getStartupMonitor();
	}

	/**
	 * @see UpdateProvider#getUpdatePath()
	 */
	@Override
	public File getUpdatePath() throws ApplicationException {
		// Ist das Unterverzeichnis "plugins" im Plugin
		return new File(manifest.getPluginDir(), "updates"); //$NON-NLS-1$
	}

	/**
	 * @see UpdateProvider#setNewVersion(int)
	 */
	@Override
	public void setNewVersion(final int newVersion) throws ApplicationException {
		final int current = getCurrentVersion();
		try {
			if (Logger.isLogging(Level.INFO)) {
				Logger.info(Settings.getI18n().tr(
						"JaDepotUpdateProvider.applyNewVerison", //$NON-NLS-1$
						this.version.getName(), Integer.toString(newVersion)));
			}
			this.version.setVersion(newVersion);
			this.version.store();
		} catch (RemoteException e) {
			// Im Fehlerfall Versionsnummer zuruecksetzen
			resetVersion(current);
			Logger.error(
					Settings.getI18n().tr(
							"JaDepotUpdateProvider.unableToReadCurrentVersion"), e); //$NON-NLS-1$
			throw (ApplicationException) new ApplicationException(
					Settings.getI18n()
							.tr(Settings
									.getI18n()
									.tr("JaDepotUpdateProvider.unableToReadCurrentVersion"))).initCause(e); //$NON-NLS-1$

		} catch (ApplicationException e) {
			// Im Fehlerfall Versionsnummer zuruecksetzen
			resetVersion(current);
			// Originalfehler zurueckgegeben
			throw e;

		}

	}

	/**
	 * Setzt Version und verwirft Fehlermeldung
	 * 
	 * @param version
	 *            die zu setzte Version.
	 */
	private void resetVersion(final int version) {
		try {
			this.version.setVersion(version);
		} catch (RemoteException e2) {
			Logger.error(
					Settings.getI18n().tr(
							"JaDepotUpdateProvider.unableToRollback"), e2); //$NON-NLS-1$
			// Werfen wir nicht, weil es sonst die eigentliche Exception
			// verdecken wuerde
		}
	}

}
