/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 */

package de.frankmuenster.jameica.depot.db.server;

import java.rmi.RemoteException;
import java.sql.Time;
import java.util.Date;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.db.rmi.OrderHistory;
import de.frankmuenster.jameica.depot.db.rmi.ValueHistory;
import de.frankmuenster.jameica.depot.db.rmi.Wertpapier;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.WertpapierInfoBean;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceAPI;
import de.frankmuenster.jameica.depot.finanzen.yahoo.finance.YahooFinanceException;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * This is the implemtor of the project interface. You never need to instanciate
 * this class directly. Instead of this, use the dbService to find the right
 * implementor of your interface. Example:
 * 
 * DBService service = (DBService)
 * Application.getServiceFactory().lookup(ExamplePlugin.class,"JaDepotDB");
 * 
 * a) create new project Project project = (Project)
 * service.createObject(Project.class,null);
 * 
 * b) load existing project with id "4". Project project = (Project)
 * service.createObject(Project.class,"4");
 * 
 * c) list existing projects DBIterator projects =
 * service.createList(Project.class);
 */

/**
 * Implementierung eines Wertpapier-Datensatzes.
 * 
 * @author munstfr
 * @version 1.0
 */
@SuppressWarnings({ "PMD.GodClass", "PMD.LongVariable" })
public class WertpapierImpl extends AbstractDBObject implements Wertpapier {

	/**
	 * Serial UID
	 */
	private static final long serialVersionUID = -4540084930902107527L;

	/**
	 * Aktien_ID ZText fr SQl-Abfragen
	 */
	private static final String AKTIEN_ID_SQL = "aktie_id = "; //$NON-NLS-1$

	/**
	 * Wertpapier in Yahoo
	 */
	private transient Boolean inYahoo;

	/**
	 * Implementierung eines Wertpapaier-Datensatzes.
	 * 
	 * @throws RemoteException
	 *             {@link java.rmi.RemoteException}
	 */
	public WertpapierImpl() throws RemoteException {
		super();
	}

		/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getAttribute(java.lang.String)
	 */
	@Override
	public Object getAttribute(final String attribute) throws RemoteException {
		Object attributObject;
		
		switch (attribute) {
		case "gesamtWert": //$NON-NLS-1$
			attributObject = getGesamtWert();
			break;
		case "anzahl": //$NON-NLS-1$
			attributObject = getAnzahl();
			break;
		default:
			attributObject = super.getAttribute(attribute);
			break;
		}

		return attributObject;
	}

	/**
	 * Sometimes you can display only one of the projects attributes (in combo
	 * boxes). Here you can define the name of this field. Please dont confuse
	 * this with the "primary KEY".
	 * 
	 * @see de.willuhn.datasource.GenericObject#getPrimaryAttribute()
	 */
	@Override
	public String getPrimaryAttribute() throws RemoteException {
		return "symbol"; //$NON-NLS-1$
	}

	/**
	 * We have to return the name of the sql table here.
	 * 
	 * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
	 */
	@Override
	protected String getTableName() {
		return "wertpapier"; //$NON-NLS-1$
	}

	/**
	 * This method will be called, before delete() is executed. Here you can
	 * make some dependency checks. If you dont want to delete the project (in
	 * case of failed dependencies) you have to throw an ApplicationException.
	 * The message of this one will be shown in users UI. So please translate
	 * the text into the users language.
	 * 
	 * @see de.willuhn.datasource.db.AbstractDBObject#deleteCheck()
	 */
	@Override
	protected void deleteCheck() throws ApplicationException {
		// maybde used later
	}

	/**
	 * This method is invoked before executing insert(). So lets check the
	 * entered data.
	 * 
	 * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
	 */
	@Override
	protected void insertCheck() throws ApplicationException {
		try {
			if (getSymbol() == null || getSymbol().length() == 0) {
				throw new ApplicationException(Settings.getI18n().tr(
						"WertpapierImpl.symbolNullOrEmpty")); //$NON-NLS-1$
			}

		} catch (RemoteException e) {
			Logger.error(
					Settings.getI18n().tr("WertpapierImpl.insertCheckFailed"), e); //$NON-NLS-1$
			throw (ApplicationException) new ApplicationException(Settings
					.getI18n().tr("WertpapierImpl.insertCheckFailedMsg")) //$NON-NLS-1$
					.initCause(e);
		}
	}

	/**
	 * This method is invoked before every update().
	 * 
	 * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
	 */
	@Override
	protected void updateCheck() throws ApplicationException {
		// we simply call the insertCheck here ;)
		insertCheck();
	}

	/**
	 * @see de.willuhn.datasource.db.AbstractDBObject#getForeignObject(java.lang.String)
	 */
	// protected Class getForeignObject(final String foreignKey) throws
	// RemoteException {
	// // We dont have any foreign keys here. Please check TaskImpl.java
	// // if you want to see an example.
	// return super.getForeignObject(foreignKey);
	// }

	/**
	 * @see Wertpapier#getName()
	 */
	@Override
	public String getName() throws RemoteException {
		// We can cast this directly to String, the method getField() knows the
		// meta data of this sql table ;)
		return (String) getAttribute("name"); // "name" ist the sql field name //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#getSymbol()
	 */
	@Override
	public String getSymbol() throws RemoteException {
		return (String) getAttribute("symbol"); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#getLastTradeDate()
	 */
	@Override
	public Date getLastTradeDate() throws RemoteException {
		return (Date) getAttribute("lastTradeDate"); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#getLastTradeTime()
	 */
	@Override
	public Time getLastTradeTime() throws RemoteException {
		return (Time) getAttribute("lastTradeTime"); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#getLastTradePriceOnly()
	 */
	@Override
	public double getLastTradePriceOnly() throws RemoteException {
		// AbstractDBObject will create a java.lang.Double.
		// We only have to cast it.
		final Double lastTradePriceOnly = (Double) getAttribute("lastTradePriceOnly"); //$NON-NLS-1$
		return lastTradePriceOnly == null || Double.isNaN(lastTradePriceOnly) ? 0.0
				: lastTradePriceOnly.doubleValue();
	}

	/**
	 * @see Wertpapier#getPreviousClose()
	 */
	@Override
	public double getPreviousClose() throws RemoteException {
		// AbstractDBObject will create a java.lang.Double.
		// We only have to cast it.
		final Double previousClose = (Double) getAttribute("previousClose"); //$NON-NLS-1$
		return previousClose == null || Double.isNaN(previousClose) ? 0.0
				: previousClose.doubleValue();
	}

	// @Override
	// public double getValue() throws RemoteException {
	// Double d = (Double) getAttribute("value");
	// return d == null || Double.isNaN(d) ? 0.0 : d.doubleValue();
	// }

	/**
	 * @see Wertpapier#setName(String)
	 */
	@Override
	public void setName(final String name) throws RemoteException {
		// Please use setField(<fieldname>,<value>) to store the data into the
		// right field.
		setAttribute("name", name); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setSymbol(String)
	 */
	@Override
	public void setSymbol(final String symbol) throws RemoteException {
		setAttribute("symbol", symbol); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setLastTradeDate(Date)
	 */
	@Override
	public void setLastTradeDate(final Date lastTradeDate)
			throws RemoteException {
		setAttribute("lastTradeDate", lastTradeDate); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setLastTradeTime(Time)
	 */
	@Override
	public void setLastTradeTime(final Time lastTradeTime)
			throws RemoteException {
		setAttribute("lastTradeTime", lastTradeTime); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setLastTradePriceOnly(double)
	 */
	@Override
	public void setLastTradePriceOnly(final double lastTradePriceOnly)
			throws RemoteException {
		setAttribute("lastTradePriceOnly", new Double(lastTradePriceOnly)); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setPreviousClose(double)
	 */
	@Override
	public void setPreviousClose(final double previousClose)
			throws RemoteException {
		setAttribute("previousClose", new Double(previousClose)); //$NON-NLS-1$
	}

	// @Override
	// public void setValue(double value) throws RemoteException {
	// setAttribute("value", new Double(value));
	// }

	/**
	 * @see Wertpapier#getValueHistory()
	 */
	@Override
	public DBIterator<?> getValueHistory() throws RemoteException {
		try {
			final DBService service = this.getService();
			final DBIterator<?> valueHistory = service
					.createList(ValueHistory.class);
			valueHistory.addFilter(AKTIEN_ID_SQL + this.getID());

			return valueHistory;
		} catch (RemoteException e) {
			throw new RemoteException(Settings.getI18n().tr(
					"WertpapierImpl.loadHistoryValuesFailed"), e); //$NON-NLS-1$
		}
	}

	/**
	 * @see Wertpapier#getOrderHistory()
	 */
	@Override
	public DBIterator<?> getOrderHistory() throws RemoteException {
		try {
			final DBService service = this.getService();
			final DBIterator<?> orderHistory = service
					.createList(OrderHistory.class);
			orderHistory.addFilter(AKTIEN_ID_SQL + this.getID());
			orderHistory.setOrder("ORDER BY orderDate desc"); //$NON-NLS-1$
			return orderHistory;
		} catch (RemoteException e) {
			throw new RemoteException(Settings.getI18n().tr(
					"WertpapierImpl.loadOrderHistoryFailed"), e); //$NON-NLS-1$
		}
	}

	/**
	 * @see Wertpapier#getComment()
	 */
	@Override
	public String getComment() throws RemoteException {
		return (String) getAttribute("comment"); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setComment(String)
	 */
	@Override
	public void setComment(final String comment) throws RemoteException {
		setAttribute("comment", comment); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#getYearLow()
	 */
	@Override
	public double getYearLow() throws RemoteException {
		final Double yearLow = (Double) getAttribute("yearLow"); //$NON-NLS-1$
		return yearLow == null || Double.isNaN(yearLow) ? 0.0 : yearLow
				.doubleValue();
	}

	/**
	 * @see Wertpapier#setYearLow(double)
	 */
	@Override
	public void setYearLow(final double yearLow) throws RemoteException {
		setAttribute("yearLow", yearLow); //$NON-NLS-1$

	}

	/**
	 * @see Wertpapier#getYearHigh()
	 */
	@Override
	public double getYearHigh() throws RemoteException {
		final Double yearHigh = (Double) getAttribute("yearHigh"); //$NON-NLS-1$
		return yearHigh == null || Double.isNaN(yearHigh) ? 0.0 : yearHigh
				.doubleValue();
	}

	/**
	 * @see Wertpapier#setYearHigh(double)
	 */
	@Override
	public void setYearHigh(final double yearHigh) throws RemoteException {
		setAttribute("yearHigh", yearHigh); //$NON-NLS-1$

	}

	/**
	 * @see Wertpapier#getChange()
	 */
	@Override
	public double getChange() throws RemoteException {
		final Double change = (Double) getAttribute("change"); //$NON-NLS-1$
		return change == null || Double.isNaN(change) ? 0.0 : change
				.doubleValue();
	}

	/**
	 * @see Wertpapier#setChange(double)
	 */
	@Override
	public void setChange(final double change) throws RemoteException {
		setAttribute("change", change); //$NON-NLS-1$

	}

	/**
	 * @see Wertpapier#getChangeInPercentage()
	 */
	@Override
	public double getChangeInPercentage() throws RemoteException {
		final Double changeInPercentage = (Double) getAttribute("changeInPercentage"); //$NON-NLS-1$
		return changeInPercentage == null || Double.isNaN(changeInPercentage) ? 0.0
				: changeInPercentage.doubleValue();
	}

	/**
	 * @see Wertpapier#setChangeInPercentage(double)
	 */
	@Override
	public void setChangeInPercentage(final double changeInPercentage)
			throws RemoteException {
		setAttribute("changeInPercentage", changeInPercentage); //$NON-NLS-1$

	}

	/**
	 * @see Wertpapier#getType()
	 */
	@Override
	public String getType() throws RemoteException {
		return (String) getAttribute("type"); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setType(String)
	 */
	@Override
	public void setType(final String type) throws RemoteException {
		setAttribute("type", type); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#getExchange()
	 */
	@Override
	public String getExchange() throws RemoteException {
		return (String) getAttribute("exchange"); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setExchange(String)
	 */
	@Override
	public void setExchange(final String exchange) throws RemoteException {
		setAttribute("exchange", exchange); //$NON-NLS-1$

	}

	/**
	 * @see Wertpapier#getCurrency()
	 */
	@Override
	public String getCurrency() throws RemoteException {
		return (String) getAttribute("currency"); //$NON-NLS-1$
	}

	/**
	 * @see Wertpapier#setCurrency(String)
	 */
	@Override
	public void setCurrency(final String currency) throws RemoteException {
		setAttribute("currency", currency); //$NON-NLS-1$

	}

	/**
	 * @see Wertpapier#parseWertpapier(WertpapierInfoBean)
	 */
	@Override
	public void parseWertpapier(final WertpapierInfoBean wpi)
			throws RemoteException {

		setChange(wpi.getChange());
		setSymbol(wpi.getSymbol());
		setCurrency(wpi.getCurrency());
		setExchange(wpi.getStockExchange());
		setLastTradeDate(wpi.getLastTradeDate());
		setLastTradeTime(wpi.getLastTradeTime());
		setLastTradePriceOnly(wpi.getLastTradePriceOnly());
		setName(wpi.getName());
		setPreviousClose(wpi.getPreviousClose());
		setChangeInPercentage(wpi.getChangeinPercent());
		setChange(wpi.getChange());
	}

	/**
	 * @see Wertpapier#getAnzahl()
	 */
	@Override
	public double getAnzahl() throws RemoteException {
		try {
			final DBService service = this.getService();
			final DBIterator<?> orderHistory = service
					.createList(OrderHistory.class);
			orderHistory.addFilter(AKTIEN_ID_SQL + this.getID());

			double anzahl = 0.0d;
			while (orderHistory.hasNext()) {
				final OrderHistory order = (OrderHistory) orderHistory.next();
				anzahl += order.getOrderAmount();
			}

			return anzahl;
		} catch (RemoteException e) {
			throw new RemoteException(Settings.getI18n().tr(
					"WertpapierImpl.calculatingNumberOfSharesFailed"), e); //$NON-NLS-1$
		}
	}

	/**
	 * @see Wertpapier#isInYahoo()
	 */
	@Override
	public boolean isInYahoo() throws RemoteException {
		if (inYahoo == null) {
			// Aktienabfrage starten
			try {
				WertpapierInfoBean wpInfo = null;
				wpInfo = YahooFinanceAPI.getInstance() // NOPMD
						.getWertpapierInfo(this.getSymbol(), true);
				// wenn wir hier sind, gibt es das Wertpapier
				if (wpInfo != null) {
					inYahoo = true;
				}
			} catch (RemoteException e) {
				inYahoo = false;
			} catch (YahooFinanceException e) {
				inYahoo = false;
			}
		}
		return inYahoo;
	}

	/**
	 * @see Wertpapier#getKauf()
	 */
	@Override
	public double getKauf() throws RemoteException {
		try {
			final DBService service = this.getService();
			final DBIterator<?> orderHistory = service
					.createList(OrderHistory.class);
			orderHistory.addFilter(AKTIEN_ID_SQL + this.getID());

			double costs = 0.0d;
			while (orderHistory.hasNext()) {
				final OrderHistory order = (OrderHistory) orderHistory.next();
				if (order.getOrderAmount() > 0) {
					costs += order.getOrderPrice() * order.getOrderAmount();
				}
			}

			return costs;
		} catch (RemoteException e) {
			throw new RemoteException(Settings.getI18n().tr(
					"WertpapierImpl.calculatingExpensesFailed"), e); //$NON-NLS-1$
		}

	}

	/**
	 * @see Wertpapier#getAdditionalCosts()
	 */
	@Override
	public double getAdditionalCosts() throws RemoteException {

		try {
			final DBService service = this.getService();
			final DBIterator<?> orderHistory = service
					.createList(OrderHistory.class);
			orderHistory.addFilter(AKTIEN_ID_SQL + this.getID());

			double additionalCosts = 0.0d;
			while (orderHistory.hasNext()) {
				final OrderHistory order = (OrderHistory) orderHistory.next();
				additionalCosts += order.getAdditionalCosts();
			}

			return additionalCosts;
		} catch (RemoteException e) {
			throw new RemoteException(Settings.getI18n().tr(
					"WertpapierImpl.calculatingAdditionalCostsFailed"), e); //$NON-NLS-1$
		}

	}

	/**
	 * @see Wertpapier#getGesamtWert()
	 */
	@Override
	public double getGesamtWert() throws RemoteException {

		return getAnzahl() * getLastTradePriceOnly();
	}

	/**
	 * @see Wertpapier#getTotalGewinn()
	 */
	@Override
	public double getTotalGewinn() throws RemoteException {
		double gewinn = 0.0d;

		final DBIterator<?> orderHistory = this.getOrderHistory();

		while (orderHistory.hasNext()) {
			final OrderHistory order = (OrderHistory) orderHistory.next();

			// double orderKurs = order.getOrderPrice();
			final double orderGewinn = (this.getLastTradePriceOnly() - order
					.getOrderPrice()) * order.getOrderAmount();

			if (Logger.isLogging(Level.DEBUG)) {
				if (order.getOrderAmount() > 0) {
					Logger.debug(Settings
							.getI18n()
							.tr("WertpapierImpl.orderProfitPurchase", Double.toString(orderGewinn))); //$NON-NLS-1$
				} else if (order.getOrderAmount() < 0) {
					Logger.debug(Settings
							.getI18n()
							.tr("WertpapierImpl.orderProfitSale", Double.toString(orderGewinn))); //$NON-NLS-1$
				}
			}
			gewinn += orderGewinn;
		}

		return gewinn;
	}

	/**
	 * @see Wertpapier#getVerkauf()
	 */
	@Override
	public double getVerkauf() throws RemoteException {
		double realisiert = 0.0d;

		final DBIterator<?> orderHistory = this.getOrderHistory();

		while (orderHistory.hasNext()) {
			final OrderHistory order = (OrderHistory) orderHistory.next();
			if (order.getOrderAmount() < 0) {
				// Nur Verkufe bercksichtigen
				realisiert += order.getOrderPrice() * order.getOrderAmount()
						* -1;
			}
		}

		return realisiert;
	}


}
