/*
 * Copyright 2016 frankmuenster.de
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   http://ec.europa.eu/idabc/eupl
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.finanzen.yahoo.finance;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.sql.Time;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;

import de.frankmuenster.jameica.depot.Settings;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;

/**
 * Schnittstelle zur Yahoo Finanz-API
 * 
 * @author munstfr
 * @version 1.0
 */
public final class YahooFinanceAPI {

	/**
	 * Yahoo Standartantwort, falls nichts gefunden wurde.
	 */
	private final static String NOT_A_RESOURCE = "NaR"; //$NON-NLS-1$

	/**
	 * Zwischenspeicher f&uuml;r schon abgefragte StockInfos
	 */
	private final transient Map<String, List<StockInfoBean>> cacheStockInfo = new ConcurrentHashMap<String, List<StockInfoBean>>();

	/**
	 * Zwischenspeicher f&uuml;r schon abgefragte WertpapierInfos
	 */
	private final transient Map<String, WertpapierInfoBean> cacheWpInfo = new ConcurrentHashMap<String, WertpapierInfoBean>();

	/**
	 * Liste der schon abgefragte URLs
	 */
	private final transient Map<URL, String> cacheUrlResponse = new ConcurrentHashMap<URL, String>();

	/**
	 * Zwischenspeicher f&uuml;r schon abgefragte WertpapierDividendenHistory
	 */
	// private final transient Map<String, WertpapierDividendenHistory>
	// cacheWertpapierDividendenHistory = new ConcurrentHashMap<String,
	// WertpapierDividendenHistory>();

	/**
	 * Unser Instanz
	 */
	private static YahooFinanceAPI instance;

	/**
	 * Privater Konstruktor
	 */
	private YahooFinanceAPI() {

	}

	/**
	 * Liefert die YahooFinanceAPI Instanz. Wird bei bedarf erstellt.
	 * 
	 * @return die YahooFinanceAPI-Instanz
	 */
	public static YahooFinanceAPI getInstance() {
		synchronized (YahooFinanceAPI.class) {
			if (YahooFinanceAPI.instance == null) {
				YahooFinanceAPI.instance = new YahooFinanceAPI();
			}
		}
		return YahooFinanceAPI.instance;
	}

	/**
	 * Liefert eine Liste mit allen Wertpapieren, die zu der angegebene Quote
	 * passen. Kann prima zum Suchen von Wertpapieren benutzt werden.
	 * 
	 * @param quote
	 *            der Suchtext.
	 * @return die Liste mit allen gefundenen Wertpapieren.
	 * @throws YahooFinanceException
	 */
	public List<StockInfoBean> lookUpQuote(final String quote)
			throws YahooFinanceException {

		URL url;
		String request = ""; //$NON-NLS-1$
		try {
			final String quoteEnc = URLEncoder.encode(quote,
					StandardCharsets.UTF_8.name());
			request = "http://autoc.finance.yahoo.com/autoc?query=" //$NON-NLS-1$
					+ quoteEnc
					// + "&callback=YAHOO.Finance.SymbolSuggest.ssCallback";
					+ "&region=EU&lang=de-DE&callback=YAHOO.Finance.SymbolSuggest.ssCallback"; //$NON-NLS-1$
			url = new URL(request);
		} catch (MalformedURLException e1) {
			final String message = Settings.getI18n().tr(
					"YahooFinanceAPI.malformedUrl", request); //$NON-NLS-1$
			throw (YahooFinanceException) new YahooFinanceException(message,
					YahooFinanceExceptionType.OTHER).initCause(e1);
		} catch (UnsupportedEncodingException e) {
			throw (YahooFinanceException) new YahooFinanceException(Settings
					.getI18n().tr("YahooFinanceAPI.encodingNotSupported"), //$NON-NLS-1$
					YahooFinanceExceptionType.OTHER).initCause(e);
		}

		List<StockInfoBean> result = null;
		if (cacheStockInfo.containsKey(quote)) {
			Logger.info(Settings.getI18n().tr(
					"YahooFinanceAPI.loadfromCacheInfo")); //$NON-NLS-1$
			result = cacheStockInfo.get(quote);
		} else {
			final String responseString = getResonseGETRequest(url);
			if (!NOT_A_RESOURCE.equals(responseString)) {
				final Pattern pattern = Pattern
						.compile("YAHOO.Finance.SymbolSuggest.ssCallback\\((.*)\\)"); //$NON-NLS-1$
				final Matcher matcher = pattern.matcher(responseString);
				if (matcher.find()) {
					final String responseJson = matcher.group(1);
					result = getStockInfoFromJSON(responseJson);
					cacheStockInfo.put(quote, result);
				}
			}
		}
		return result;
	}

	/**
	 * Liefert eine Liste von WertpapierInfos zu angegeben Wertpapieren.
	 * 
	 * @param wpis
	 *            die Wertpapiere deren Infos gesucht werden sollen.
	 * @param useCache
	 *            soll der Cache verwendet werden.
	 * @return eine Liste von WertpapierInfos
	 */
	public List<WertpapierInfoBean> getWertpapierInfos(
			final List<WertpapierInfoBean> wpis, final Boolean useCache) {

		final List<WertpapierInfoBean> wpiResults = new ArrayList<WertpapierInfoBean>();

		for (final WertpapierInfoBean wpi : wpis) {
			try {
				final WertpapierInfoBean wpiTmp = getWertpapierInfo(
						wpi.getSymbol(), useCache);
				if (wpiTmp == null) {
					wpiResults.add(wpi);
				} else {
					wpiResults.add(wpiTmp);
				}

			} catch (YahooFinanceException e) {
				wpi.setNotes(Settings.getI18n().tr(
						"YahooFinanceAPI.loadStockInfoFailed")); //$NON-NLS-1$
				wpiResults.add(wpi);
			}
		}

		return wpiResults;
	}

	/**
	 * Liefert Detailangaben zu einem Wertpapier. In der aktuellen Version
	 * werden nicht alle Felder ausgelesen.
	 * 
	 * @param quote
	 *            das Yahoo-Symbol des Wertpapiers.
	 * @param useCache
	 *            Soll der interne Cache verwendet oder eine neue Anfrage an
	 *            Yahoo gesendet werden.
	 * @return ein {@link WertpapierInfoBean} Objekt mit den Details.
	 * @throws YahooFinanceException
	 *             falls die Abfrage fehlgeschlagen ist. Die Exception
	 *             enth&auml;lt weiter Hinweise.
	 */
	public WertpapierInfoBean getWertpapierInfo(final String quote,
			final Boolean useCache) throws YahooFinanceException {

		URL url;
		String request = ""; //$NON-NLS-1$
		try {
			request = "http://query.yahooapis.com/v1/public/yql?q=select%20*%20from%20yahoo.finance.quotes%20where%20symbol%20in%20%28%22" //$NON-NLS-1$
					+ URLEncoder.encode(quote, "UTF-8") //$NON-NLS-1$
					+ "%22%29&env=store%3A%2F%2Fdatatables.org%2Falltableswithkeys"; //$NON-NLS-1$
					//+ "%22%29&env=http://datatables.org/alltables.env"; //$NON-NLS-1$
			
			url = new URL(request);
		} catch (MalformedURLException e1) {
			final String message = Settings.getI18n().tr(
					"YahooFinanceAPI.malformedUrl", request); //$NON-NLS-1$
			throw (YahooFinanceException) new YahooFinanceException(message,
					YahooFinanceExceptionType.OTHER).initCause(e1);
		} catch (UnsupportedEncodingException e) {
			throw (YahooFinanceException) new YahooFinanceException(Settings
					.getI18n().tr("YahooFinanceAPI.encodingNotSuportted"), //$NON-NLS-1$
					YahooFinanceExceptionType.OTHER).initCause(e);
		}

		WertpapierInfoBean result;
		if (useCache && cacheWpInfo.containsKey(quote)) {
			Logger.info(Settings.getI18n().tr(
					"YahooFinanceAPI.loadfromCacheInfo")); //$NON-NLS-1$
			result = cacheWpInfo.get(quote);
		} else {
			if (NOT_A_RESOURCE.equals(url)) {
				final String message = Settings.getI18n().tr(
						"YahooFinanceAPI.responseIsNaR", quote); //$NON-NLS-1$
				Logger.error(message);
				throw new YahooFinanceException(message,
						YahooFinanceExceptionType.OTHER);
			}
			try {
				final String responseString = getResonseGETRequest(url);
				Logger.debug(Settings.getI18n().tr(
						"YahooFinanceAPI.responseString", responseString)); //$NON-NLS-1$
				result = getWertpapierInfoFromJson(responseString);
				cacheWpInfo.put(quote, result);
			} catch (NumberFormatException e) {
				final String message = Settings.getI18n().tr(
						"YahooFinanceAPI.parsingError" //$NON-NLS-1$
						, quote);
				if (Logger.isLogging(Level.DEBUG)) {
					Logger.error(message, e);
				} else {
					Logger.error(message);
				}
				throw (YahooFinanceException) new YahooFinanceException(
						message, YahooFinanceExceptionType.OTHER).initCause(e);
			}
		}

		return result;
	}

	private WertpapierInfoBean getWertpapierInfoFromJson(final String json) {

		final WertpapierInfoBean resultQuote = new WertpapierInfoBean();

		final JSONObject responseObject = (JSONObject) JSONValue.parse(json);
		final JSONObject resultSet = (JSONObject) responseObject.get("query"); //$NON-NLS-1$
		final JSONObject results = (JSONObject) resultSet.get("results"); //$NON-NLS-1$
		final JSONObject quoteInfoObj = (JSONObject) results.get("quote"); //$NON-NLS-1$

		final DateFormat dateFormat = new SimpleDateFormat("M/d/yyyy", //$NON-NLS-1$
				Locale.GERMAN);
		final DateFormat timeFormat = new SimpleDateFormat("h:mma", //$NON-NLS-1$
				Locale.GERMAN);

		resultQuote.setSymbol((String) quoteInfoObj.get("Symbol")); //$NON-NLS-1$
		resultQuote.setName((String) quoteInfoObj.get("Name")); //$NON-NLS-1$
		resultQuote
				.setStockExchange((String) quoteInfoObj.get("StockExchange")); //$NON-NLS-1$
		resultQuote.setChange(Double.parseDouble((String) quoteInfoObj
				.get("Change"))); //$NON-NLS-1$
		resultQuote.setPercentChange(fromPercentageString((String) quoteInfoObj
				.get("PercentChange"))); //$NON-NLS-1$
		resultQuote
				.setChangeinPercent(fromPercentageString((String) quoteInfoObj
						.get("PercentChange"))); //$NON-NLS-1$
		resultQuote.setLastTradePriceOnly(Double
				.parseDouble((String) quoteInfoObj.get("LastTradePriceOnly"))); //$NON-NLS-1$
		resultQuote.setCurrency(((String) quoteInfoObj.get("Currency")) //$NON-NLS-1$
				.toUpperCase());
		try {
			resultQuote.setLastTradeDate(dateFormat.parse((String) quoteInfoObj
					.get("LastTradeDate"))); //$NON-NLS-1$
			final String ltAsString = (String) quoteInfoObj
					.get("LastTradeTime"); //$NON-NLS-1$
			final Time lastTradeTime = new Time(timeFormat.parse(ltAsString)
					.getTime());
			resultQuote.setLastTradeTime(lastTradeTime);
			if (Logger.getLevel().equals(Level.INFO)) {
				Logger.info(Settings
						.getI18n()
						.tr("YahooFinanceAPI.time", timeFormat.format(lastTradeTime))); // NOPMD //$NON-NLS-1$
			}
		} catch (ParseException e) {
			Logger.warn(Settings.getI18n().tr(
					"YahooFinanceAPI.readingLastTradeDateFaild")); //$NON-NLS-1$
		}
		resultQuote.setPreviousClose(Double.parseDouble((String) quoteInfoObj
				.get("PreviousClose"))); //$NON-NLS-1$

		return resultQuote;
	}

	private List<StockInfoBean> getStockInfoFromJSON(final String json) {
		final List<StockInfoBean> tickers = new ArrayList<StockInfoBean>();
		final JSONObject responseObject = (JSONObject) JSONValue.parse(json);
		final JSONObject resultSet = (JSONObject) responseObject
				.get("ResultSet"); //$NON-NLS-1$
		final JSONArray results = (JSONArray) resultSet.get("Result"); //$NON-NLS-1$
		for (final Object tickerInfo : results) {
			final JSONObject tickerInfoObj = (JSONObject) tickerInfo;
			final StockInfoBean stockTicker = new StockInfoBean(); // NOPMD
			stockTicker.setSymbol((String) tickerInfoObj.get("symbol")); //$NON-NLS-1$
			stockTicker.setName((String) tickerInfoObj.get("name")); //$NON-NLS-1$
			stockTicker.setExchange((String) tickerInfoObj.get("exchDisp")); //$NON-NLS-1$
			stockTicker.setType((String) tickerInfoObj.get("typeDisp")); //$NON-NLS-1$
			tickers.add(stockTicker);
		}
		return tickers;
	}

	/**
	 * Liefert die Antwort eines Web-Services Aufrufs.
	 * 
	 * @param url
	 *            des Webservices.
	 * @return die Antwort des Web Service, oder <code>"NaR"</code> falls mit
	 *         dem Aufruf alles in Ordnung war, aber keine Antwort kam.
	 * @throws YahooFinanceException
	 *             falls w&auml;hrend des Aufrufs etwas schief ging.
	 */
	private String getResonseGETRequest(final URL url)
			throws YahooFinanceException {
		return getResonseGETRequest(url, false);
	}

	/**
	 * Lieftert die Antwort eines Web-Services Aufrufs.
	 * 
	 * @param url
	 *            des Webservices.
	 * @param useCache
	 *            <code>true</code> falls der interne Cache verwendet werden
	 *            soll, sonst <code>false</code>.
	 * @return die Antwort des Web Service, oder <code>"NaR"</code> falls mit
	 *         derm Aufruif alles in Ordnung war, aber keine Antwort kam.
	 * @throws YahooFinanceException
	 *             falls w&auml;hrend des Aufrufs etwas schief ging.
	 */
	public String getResonseGETRequest(final URL url, final boolean useCache)
			throws YahooFinanceException {

		String response;

		if (useCache && cacheUrlResponse.containsKey(url)) {
			Logger.debug(Settings.getI18n().tr(
					"YahooFinanceAPI.loadfromCacheInfo")); //$NON-NLS-1$
			response = cacheUrlResponse.get(url);
		} else {

			response = NOT_A_RESOURCE;
			try {
				if (Level.DEBUG.equals(Logger.getLevel())) {
					Logger.debug(Settings.getI18n().tr(
							"YahooFinanceAPI.url", url.toString())); //$NON-NLS-1$
				}
				final HttpURLConnection conn = (HttpURLConnection) url
						.openConnection();

				conn.setRequestMethod("GET"); //$NON-NLS-1$
				conn.setConnectTimeout(2000);
				conn.setRequestProperty("Accept", "application/json"); //$NON-NLS-1$ //$NON-NLS-2$
				conn.setRequestProperty("Accept-Charset", "UTF-8"); //$NON-NLS-1$ //$NON-NLS-2$

				if (conn.getResponseCode() != HttpURLConnection.HTTP_OK) {
					throw new YahooFinanceException(Settings.getI18n().tr(
							"YahooFinanceAPI.httpError" //$NON-NLS-1$
							, Integer.toString(conn.getResponseCode())),
							YahooFinanceExceptionType.NETWORK);
				}

				final BufferedReader bufferReader = new BufferedReader(
						new InputStreamReader(conn.getInputStream(), "UTF-8")); //$NON-NLS-1$

				Logger.warn(Settings.getI18n().tr(
						"YahooFinanceAPI.serverOutput")); //$NON-NLS-1$
				// System.out.println("Output from Server .... \n");
				final StringBuffer sBuffer = new StringBuffer(""); //$NON-NLS-1$
				String output;
				while ((output = bufferReader.readLine()) != null) { // NOPMD
					// System.out.println(output);
					sBuffer.append(output).append(System.lineSeparator());
				}

				response = sBuffer.toString();
				conn.disconnect();

				if (useCache) {
					cacheUrlResponse.put(url, response);
				}

			} catch (MalformedURLException e) {
				throw (YahooFinanceException) new YahooFinanceException(
						Settings.getI18n().tr(
								"YahooFinanceAPI.malformedUrl", e.getMessage()), //$NON-NLS-1$
						YahooFinanceExceptionType.NETWORK).initCause(e);

			} catch (IOException e) {
				throw (YahooFinanceException) new YahooFinanceException(
						Settings.getI18n().tr(
								"YahooFinanceAPI.ioException", e.getMessage()), //$NON-NLS-1$
						YahooFinanceExceptionType.NETWORK).initCause(e);
			}
		}
		return response;

	}

	private double fromPercentageString(final String value) {
		String replaceValue = value.replace("+", ""); //$NON-NLS-1$ //$NON-NLS-2$
		replaceValue = replaceValue.replace("%", ""); //$NON-NLS-1$ //$NON-NLS-2$
		return Double.parseDouble(replaceValue);
	}
}
