/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.gui.input;

import java.util.Date;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Control;
import org.mihalis.opal.rangeSlider.RangeSlider;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.gui.utils.DateUtils;
import de.willuhn.jameica.gui.input.AbstractInput;

/**
 * Ein Schieber mit 2 Markern zur Auswahl eines Zeitbereiches.
 * 
 * @author munstfr
 * @version 1.0
 */
public class ZeitauswahlSchieberInput extends AbstractInput {

	/**
	 * Haben wir den Fokus
	 */
	private transient Boolean hasFocus = false;

	/**
	 * Sind wir Enabled
	 */
	private boolean enabled = true;

	/**
	 * Minimaler Wert
	 */
	private transient int minimun;

	/**
	 * Maximaler Wert. Standard 1 Jahr (365 Tage).
	 */
	private transient int maximum = 365;

	/**
	 * Position des unteren Schiebers
	 */
	private int lower;

	/**
	 * Position des oberen Schiebers
	 */
	private int upper;

	/**
	 * Der RangeSlider
	 */
	private transient RangeSlider slider;

	/**
	 * Die Listener fr ein ZeitauswahlSchieberInputListener-Event
	 */
	private final transient Set<ZeitauswahlSchieberInputListener> listeners = new CopyOnWriteArraySet<ZeitauswahlSchieberInputListener>();

	/**
	 * Fgt eine Listener hinzu
	 * 
	 * @param listener
	 *            der hinzugefgt werden soll
	 */
	public final void addListener(
			final ZeitauswahlSchieberInputListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernd einen Listener aus der Liste
	 * 
	 * @param listener
	 *            der entfernt werden soll
	 */
	public final void removeListener(
			final ZeitauswahlSchieberInputListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Benachrichtigen der registrieretn Listener
	 */
	public final void notifyListeners() {
		// erst mal unseren Kommentar aktualisieren
		updateComment();
		for (final ZeitauswahlSchieberInputListener listener : listeners) {
			listener.rangeChanged();
		}
	}

	/**
	 * Default-Konstruktor mit default Werten.
	 */
	public ZeitauswahlSchieberInput() {
		this(DateUtils.subtractDays(365), new Date());
	}

	/**
	 * Default-Konstruktor.
	 * 
	 * @param lower
	 *            unteres Datum.
	 * @param upper
	 *            oberes Datum.
	 */
	public ZeitauswahlSchieberInput(final Date lower, final Date upper) {

		super();
		setLower(lower);
		setUpper(upper);
	}

	/**
	 * @return lower
	 */
	public Date getLower() {
		return DateUtils.subtractDays(this.maximum - this.lower);
	}

	/**
	 * @param lower
	 *            das zu setzende Objekt lower
	 */
	public final void setLower(final Date lower) {
		final int low = (int) DateUtils.dateDiff(lower);
		setLower(low);
	}

	/**
	 * @param lower
	 *            das zu setzende Objekt lower
	 */
	public void setLower(final int lower) {
		this.lower = lower > this.minimun & lower < this.upper ? lower : 0;
		if (slider != null && this.lower != slider.getLowerValue()) {
			slider.setLowerValue(this.lower);
			updateComment();
		}
	}

	/**
	 * @return upper
	 */
	public Date getUpper() {
		return DateUtils.subtractDays(this.maximum - this.upper);
	}

	/**
	 * @param upper
	 *            das zu setzende Objekt upper
	 */
	public final void setUpper(final Date upper) {
		final int upperDay = (int) DateUtils.dateDiff(upper);
		setUpper(upperDay);
	}

	/**
	 * @param upper
	 *            das zu setzende Objekt upper
	 */
	public void setUpper(final int upper) {
		this.upper = upper < this.maximum & upper > this.lower ? upper
				: this.maximum;
		if (slider != null && this.upper != slider.getUpperValue()) {
			slider.setUpperValue(this.upper);
			updateComment();
		}
	}

	/**
	 * Liefert den maximalen Wert des RangeSliders.
	 * 
	 * @return dem Maximalwert
	 */
	public int getMaximun() {
		return this.maximum;
	}

	/**
	 * Liefert den minimalen Wert des RangeSliders.
	 * 
	 * @return dem Minimalwert
	 */
	public int getMinimum() {
		return this.minimun;
	}

	/**
	 * Setzt die Werte f&Uuml;r Skalierung und Schrittweite.
	 * 
	 * @param fromDate
	 *            Minimum-Wert (Default: 0).
	 * @param toDate
	 *            Maximum-Wert (Default: 100).
	 * @param increment
	 *            Schrittweite (Default: 1).
	 * @param pageIncrement
	 *            seitenweise Schrittweite (Default: 10).
	 */
	public void setScaling(final Date fromDate, final Date toDate,
			final int increment, final int pageIncrement) {

		this.minimun = 0;
		this.maximum = (int) DateUtils.dateDiff(fromDate, toDate);
		if (this.slider != null && !slider.isDisposed()) {
			this.slider.setMinimum(this.minimun);
			this.slider.setMaximum(this.maximum);
			this.slider.setIncrement(increment);
			this.slider.setPageIncrement(pageIncrement);
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#getValue()
	 */
	@Override
	public Object getValue() {
		return new int[] { this.lower, this.upper };
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#setValue(java.lang.Object)
	 */
	@Override
	public void setValue(final Object value) {
		// check if int array with two values
		if (value instanceof int[] && ((int[]) value).length == 2) {
			// OK, zwei int Werte, dann versuchen wir es mal
			this.setLower(((int[]) value)[0]);
			this.setUpper(((int[]) value)[1]);
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#getControl()
	 */
	@Override
	public Control getControl() {

		if (this.slider == null || this.slider.isDisposed()) {

			this.slider = new RangeSlider(getParent(), SWT.HORIZONTAL);
			final GridData gridData = new GridData(GridData.FILL,
					GridData.CENTER, true, false, 1, 2);
			gridData.widthHint = 250;
			this.slider.setLayoutData(gridData);
			this.slider.setMinimum(this.minimun);
			this.slider.setMaximum(this.maximum);
			this.slider.setUpperValue(this.upper);
			this.slider.setLowerValue(this.lower);

			final Date fromDate = getLower();
			final Date toDate = getUpper();
			final String lString = Settings.DATEFORMAT.format(fromDate);
			final String uString = Settings.DATEFORMAT.format(toDate);
			final int tage = (int) DateUtils.dateDiff(fromDate, toDate);

			setComment(createCommentText(lString, uString, tage));

			final ZeitauswahlSchieberInputChangedListener changed = new ZeitauswahlSchieberInputChangedListener(
					this);

			this.slider.addSelectionListener(changed);
			this.slider.addMouseMoveListener(changed);
			this.slider.addKeyListener(changed);

			if (this.hasFocus) {
				this.slider.setFocus();
			}
		}
		return this.slider;

	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#focus()
	 */
	@Override
	public void focus() {
		this.hasFocus = true;
		if (this.slider != null && !this.slider.isDisposed()) {
			this.slider.setFocus();
		}
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#disable()
	 */
	@Override
	public void disable() {
		this.setEnabled(false);
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#enable()
	 */
	@Override
	public void enable() {
		this.setEnabled(true);
	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#setEnabled(boolean)
	 */
	@Override
	public void setEnabled(final boolean enabled) {
		this.enabled = enabled;
		if (this.slider != null && !this.slider.isDisposed()) {
			this.slider.setEnabled(this.enabled);
		}

	}

	/**
	 * @see de.willuhn.jameica.gui.input.Input#isEnabled()
	 */
	@Override
	public boolean isEnabled() {
		return this.enabled;
	}

	private void updateComment() {

		final int lValue = this.slider.getLowerValue();
		final int uValue = this.slider.getUpperValue();

		final Date fromDate = DateUtils.subtractDays(maximum - lValue);
		final Date toDate = DateUtils.subtractDays(maximum - uValue);

		final String lString = Settings.DATEFORMAT.format(fromDate);
		final String uString = Settings.DATEFORMAT.format(toDate);
		final int tage = (int) DateUtils.dateDiff(fromDate, toDate);

		setComment(createCommentText(lString, uString, tage));
	}

	/**
	 * Auswahl des letzten Jahren (sprich 365 Tage)
	 */
	public void selectLastYear() {
		this.lower = this.maximum > 365 ? this.maximum - 365 : 0;
		this.upper = this.maximum;

		if (slider != null && !slider.isDisposed()) {
			slider.setLowerValue(this.lower);
			slider.setUpperValue(this.maximum);

			final Date fromDate = getLower();
			final Date toDate = getUpper();

			final String lString = Settings.DATEFORMAT.format(fromDate);
			final String uString = Settings.DATEFORMAT.format(toDate);
			final int tage = (int) DateUtils.dateDiff(fromDate, toDate);

			setComment(createCommentText(lString, uString, tage));
			notifyListeners();
		}

	}

	/**
	 * Erstellt den Kommentartext.
	 * 
	 * @param lValue
	 * @param uValue
	 * @param days
	 * @return den Text
	 */
	private String createCommentText(final String lValue, final String uValue, final int days) {
		String comment;
		if (days == 1) { // NOPMD
			comment = Settings
					.getI18n()
					.tr("ZeitauswahlSchieberInput.commentTextDays", lValue, uValue, Integer.toString(days)); //$NON-NLS-1$
		} else {
			comment = Settings
					.getI18n()
					.tr("ZeitauswahlSchieberInput.commentTextDay", lValue, uValue, Integer.toString(days)); //$NON-NLS-1$
		}
		return comment;
	}

}
