/*
 * Copyright 2016 frankmuenster.de
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie drfen dieses Werk ausschlielich gem
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 * http://ec.europa.eu/idabc/eupl
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschrnkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.frankmuenster.jameica.depot.kursanalyse;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.frankmuenster.jameica.depot.Settings;
import de.frankmuenster.jameica.depot.kursanalyse.BollingBaender.BollingBaender;
import de.frankmuenster.jameica.depot.kursanalyse.EMA.ExponentialMovingAverage;
import de.frankmuenster.jameica.depot.kursanalyse.MACD.MACD;
import de.frankmuenster.jameica.depot.kursanalyse.MOM.Momentum;
import de.frankmuenster.jameica.depot.kursanalyse.RSI.RelativeStrengthIndex;
import de.frankmuenster.jameica.depot.kursanalyse.SMA.SimpleMovingAverage;
import de.frankmuenster.jameica.depot.kursanalyse.Volumen.Volumen;

/**
 * Alle Analysemethoden als Liste.
 * 
 * @author munstfr
 * @version 1.0
 * @param <P>
 * @param <R>
 * 
 */
public final class Analysen<P, R> extends ArrayList<IKursAnalyse<P, R>> {

	/**
	 * die serial UID
	 */
	private static final long serialVersionUID = -2102978330463004776L;

	/**
	 * Die Liste mit allen verf&uuml;gbaren Analysemethoden
	 */
	private static List<Class<?>> methods;

	/**
	 * Unser Instanz
	 */
	private static Analysen<Object, Object> instance;

	/**
	 * Privater Konstruktor zum Vermeiden von mehrfache Instantiierung.
	 */
	private Analysen() {
		super();
	}

	/**
	 * Liefert eine SingletonObject der Klasse
	 * 
	 * @return das SingletonObject der Klasse
	 */
	public static Analysen<Object, Object> getInstance() {

		synchronized (Analysen.class) {
			if (instance == null) {
				instance = new Analysen<Object, Object>();
			}
		}
		return instance;

	}

	/**
	 * @return the anzahlBarAnalysen
	 */
	public int getAnzahlSeperateAnalysen() {
		int numSeperateAna = 0;
		for (final Iterator<IKursAnalyse<P, R>> iterator = super.iterator(); iterator
				.hasNext();) {
			final IKursAnalyse<P, R> type = iterator.next();
			if (type.getType().equals(KursAnalyseTyp.SEPERATE_GRAPH)) {
				numSeperateAna++;
			}
		}
		return numSeperateAna;
	}

	/**
	 * @see java.util.ArrayList#add(java.lang.Object)
	 */
	@Override
	public boolean add(final IKursAnalyse<P, R> iKursAnalyse) {
		boolean alreadyAdded = false;
		for (final IKursAnalyse<P, R> analyse : this) {
			if (iKursAnalyse.getLegendTitel().equals(analyse.getLegendTitel())) {
				alreadyAdded = true;
				break;
			}
		}
		return alreadyAdded ? false : super.add(iKursAnalyse);
	}

	/**
	 * 
	 * @see java.util.ArrayList#add(int, java.lang.Object)
	 */
	@Override
	public void add(final int index, final IKursAnalyse<P, R> iKursAnalyse) {
		boolean alreadyAdded = false;
		for (final IKursAnalyse<P, R> analyse : this) {
			if (iKursAnalyse.getLegendTitel().equals(analyse.getLegendTitel())) {
				alreadyAdded = true;
				break;
			}
		}
		if (!alreadyAdded) {
			super.add(index, iKursAnalyse);
		}
	}

	/**
	 * Liefert <i>true</i> falls es eine Analyse mit angegeben Titel gibt, sonst
	 * <i>false</i>.
	 * 
	 * @param legendTitel
	 *            der Legendentitel, nach dem gesucht werden soll.
	 * @return <i>true</i> falls es eine Analyse mit angegeben Titel gibt, sonst
	 *         <i>false</i>
	 */
	public boolean conatinsAnalyseWithLegendTitel(final String legendTitel) {

		boolean analysePresent = false;
		for (final IKursAnalyse<P, R> analyse : this) {
			if (analyse.getLegendTitel().equals(legendTitel)) {
				analysePresent = true;
				break;
			}
		}

		return analysePresent;

	}

	/**
	 * Liefert die Analyse zum Angegebenen Titel.
	 * 
	 * @param legendTitel
	 * @return die Analyse
	 * @throws AnalyseNotInListeException
	 */
	public IKursAnalyse<P, R> getAnalyseWithLegendTitel(final String legendTitel)
			throws AnalyseNotInListeException {

		for (final IKursAnalyse<P, R> analyse : this) {
			if (analyse.getLegendTitel().equals(legendTitel)) {
				return analyse;
			}
		}

		throw new AnalyseNotInListeException(Settings.getI18n().tr(
				"Analysen.analysisNotFound", legendTitel)); //$NON-NLS-1$
	}

	/**
	 * Statische Methode die eine Liste aller Analysemethoden liefert.
	 * 
	 * @return Liste aller Analysemethoden
	 */
	public static List<Class<?>> getMethods() {

		synchronized (Analysen.class) {
			if (methods == null) {
				methods = new ArrayList<Class<?>>();
				methods.add(SimpleMovingAverage.class);
				methods.add(ExponentialMovingAverage.class);
				methods.add(BollingBaender.class);
				methods.add(MACD.class);
				methods.add(Momentum.class);
				methods.add(Volumen.class);
				methods.add(RelativeStrengthIndex.class);
			}
		}
		return methods;
	}
}
